// Copyright (c) 2023 the RoughPy Developers. All rights reserved.
//
// Redistribution and use in source and binary forms, with or without modification,
// are permitted provided that the following conditions are met:
//
// 1. Redistributions of source code must retain the above copyright notice,
// this list of conditions and the following disclaimer.
//
// 2. Redistributions in binary form must reproduce the above copyright notice,
// this list of conditions and the following disclaimer in the documentation
// and/or other materials provided with the distribution.
//
// 3. Neither the name of the copyright holder nor the names of its contributors
// may be used to endorse or promote products derived from this software without
// specific prior written permission.
//
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
// LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
// SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
// OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE
// USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

//
// Created by sam on 28/07/23.
//

#ifndef ROUGHPY_PLATFORM_THREADS_H
#define ROUGHPY_PLATFORM_THREADS_H

#include <roughpy/core/alloc.h>
#include <roughpy/core/macros.h>
#include <roughpy/core/types.h>

#define RPY_ALLOW_THREADING
#if defined(_OPENMP) && defined(RPY_ALLOW_THREADING)
#  define RPY_THREADING_OPENMP 1
#else
#undef RPY_ALLOW_THREADING
#endif

namespace rpy {
namespace platform {

enum struct ThreadBackend
{
    Disabled = 0,
    OpenMP = 1
};

struct ThreadState {
    /// The threading library responsible for managing multi-threaded
    /// computation.
    ThreadBackend backend;

    /// The maximum number of threads that can be spawned by a single operation.
    int max_threads;

    /// The maximum possible number of threads that can be spawned
    int max_available_threads;

    /// Is multithreading enabled
    bool is_enabled;
};

RPY_NO_DISCARD RPY_EXPORT ThreadState get_thread_state();

RPY_NO_DISCARD RPY_EXPORT constexpr bool threading_available()
{
#ifndef RPY_ALLOW_THREADING
    return false;
#else
    return true;
#endif
}

RPY_EXPORT void set_num_threads(int num_threads);

RPY_EXPORT void set_threading_enabled(bool state);

}// namespace platform
}// namespace rpy

#endif// ROUGHPY_PLATFORMS_THREADS_H
